@@ -28,14 +28,14 @@ def skip(message, **kwargs):
     return skipper
 
 
-def missing_format(checker):
+def missing_format(Validator):
     def missing_format(test):  # pragma: no cover
         schema = test.schema
         if (
             schema is True
             or schema is False
             or "format" not in schema
-            or schema["format"] in checker.checkers
+            or schema["format"] in Validator.FORMAT_CHECKER.checkers
             or test.valid
         ):
             return
@@ -91,9 +91,7 @@ if sys.version_info < (3, 9):  # pragma: no cover
     allowed_leading_zeros = skip(
         message=message,
         subject="ipv4",
-        description=(
-            "leading zeroes should be rejected, as they are treated as octals"
-        ),
+        description="invalid leading zeroes, as they are treated as octals",
     )
 else:
     def allowed_leading_zeros(test):  # pragma: no cover
@@ -152,14 +150,15 @@ TestDraft3 = DRAFT3.to_unittest_testcase(
     DRAFT3.optional_tests_of(name="non-bmp-regex"),
     DRAFT3.optional_tests_of(name="zeroTerminatedFloats"),
     Validator=jsonschema.Draft3Validator,
-    format_checker=jsonschema.draft3_format_checker,
+    format_checker=jsonschema.Draft3Validator.FORMAT_CHECKER,
     skip=lambda test: (
         narrow_unicode_build(test)
-        or missing_format(jsonschema.draft3_format_checker)(test)
+        or missing_format(jsonschema.Draft3Validator)(test)
         or complex_email_validation(test)
         or skip(
-            message=bug(371),
+            message=bug(),
             subject="ref",
+            valid=False,
             case_description=(
                 "$ref prevents a sibling id from changing the base uri"
             ),
@@ -176,12 +175,12 @@ TestDraft4 = DRAFT4.to_unittest_testcase(
     DRAFT4.optional_tests_of(name="non-bmp-regex"),
     DRAFT4.optional_tests_of(name="zeroTerminatedFloats"),
     Validator=jsonschema.Draft4Validator,
-    format_checker=jsonschema.draft4_format_checker,
+    format_checker=jsonschema.Draft4Validator.FORMAT_CHECKER,
     skip=lambda test: (
         narrow_unicode_build(test)
         or allowed_leading_zeros(test)
         or leap_second(test)
-        or missing_format(jsonschema.draft4_format_checker)(test)
+        or missing_format(jsonschema.Draft4Validator)(test)
         or complex_email_validation(test)
         or skip(
             message=bug(),
@@ -189,7 +188,7 @@ TestDraft4 = DRAFT4.to_unittest_testcase(
             case_description="Recursive references between schemas",
         )(test)
         or skip(
-            message=bug(371),
+            message=bug(),
             subject="ref",
             case_description=(
                 "Location-independent identifier with "
@@ -197,19 +196,19 @@ TestDraft4 = DRAFT4.to_unittest_testcase(
             ),
         )(test)
         or skip(
-            message=bug(371),
+            message=bug(),
             subject="ref",
             case_description=(
                 "$ref prevents a sibling id from changing the base uri"
             ),
         )(test)
         or skip(
-            message=bug(371),
+            message=bug(),
             subject="id",
             description="match $ref to id",
         )(test)
         or skip(
-            message=bug(371),
+            message=bug(),
             subject="id",
             description="no match on enum or $ref to id",
         )(test)
@@ -237,35 +236,18 @@ TestDraft6 = DRAFT6.to_unittest_testcase(
     DRAFT6.optional_tests_of(name="float-overflow"),
     DRAFT6.optional_tests_of(name="non-bmp-regex"),
     Validator=jsonschema.Draft6Validator,
-    format_checker=jsonschema.draft6_format_checker,
+    format_checker=jsonschema.Draft6Validator.FORMAT_CHECKER,
     skip=lambda test: (
         narrow_unicode_build(test)
         or allowed_leading_zeros(test)
         or leap_second(test)
-        or missing_format(jsonschema.draft6_format_checker)(test)
+        or missing_format(jsonschema.Draft6Validator)(test)
         or complex_email_validation(test)
-        or skip(
-            message="id is incorrectly finding non-ids",
-            subject="id",
-            description="const at const_not_anchor does not match",
-        )(test)
-        or skip(
-            message="id is incorrectly finding non-ids",
-            subject="id",
-            description="const at const_not_id does not match",
-        )(test)
         or skip(
             message=bug(),
             subject="refRemote",
             case_description="base URI change - change folder in subschema",
         )(test)
-        or skip(
-            message=bug(371),
-            subject="ref",
-            case_description=(
-                "$ref prevents a sibling $id from changing the base uri"
-            ),
-        )(test)
     ),
 )
 
@@ -274,29 +256,22 @@ TestDraft7 = DRAFT7.to_unittest_testcase(
     DRAFT7.tests(),
     DRAFT7.format_tests(),
     DRAFT7.optional_tests_of(name="bignum"),
-    DRAFT7.optional_tests_of(name="content"),
+    DRAFT7.optional_tests_of(name="cross-draft"),
     DRAFT7.optional_tests_of(name="float-overflow"),
     DRAFT7.optional_tests_of(name="non-bmp-regex"),
     Validator=jsonschema.Draft7Validator,
-    format_checker=jsonschema.draft7_format_checker,
+    format_checker=jsonschema.Draft7Validator.FORMAT_CHECKER,
     skip=lambda test: (
         narrow_unicode_build(test)
         or allowed_leading_zeros(test)
         or leap_second(test)
-        or missing_format(jsonschema.draft7_format_checker)(test)
+        or missing_format(jsonschema.Draft7Validator)(test)
         or complex_email_validation(test)
         or skip(
             message=bug(),
             subject="refRemote",
             case_description="base URI change - change folder in subschema",
         )(test)
-        or skip(
-            message=bug(371),
-            subject="ref",
-            case_description=(
-                "$ref prevents a sibling $id from changing the base uri"
-            ),
-        )(test)
         or skip(
             message=bug(),
             subject="ref",
@@ -305,28 +280,6 @@ TestDraft7 = DRAFT7.to_unittest_testcase(
                 "not just immediate parent"
             ),
         )(test)
-        or skip(
-            message=bug(593),
-            subject="content",
-            valid=False,
-            case_description=(
-                "validation of string-encoded content based on media type"
-            ),
-        )(test)
-        or skip(
-            message=bug(593),
-            subject="content",
-            valid=False,
-            case_description="validation of binary string-encoding",
-        )(test)
-        or skip(
-            message=bug(593),
-            subject="content",
-            valid=False,
-            case_description=(
-                "validation of binary-encoded media type documents"
-            ),
-        )(test)
     ),
 )
 
@@ -334,39 +287,106 @@ TestDraft7 = DRAFT7.to_unittest_testcase(
 TestDraft201909 = DRAFT201909.to_unittest_testcase(
     DRAFT201909.tests(),
     DRAFT201909.optional_tests_of(name="bignum"),
+    DRAFT201909.optional_tests_of(name="cross-draft"),
     DRAFT201909.optional_tests_of(name="float-overflow"),
     DRAFT201909.optional_tests_of(name="non-bmp-regex"),
     DRAFT201909.optional_tests_of(name="refOfUnknownKeyword"),
     Validator=jsonschema.Draft201909Validator,
     skip=lambda test: (
         skip(
-            message="unevaluatedItems is different in 2019-09 (needs work).",
-            subject="unevaluatedItems",
-        )(test)
-        or skip(
-            message="dynamicRef support isn't working yet.",
+            message="recursiveRef support isn't working yet.",
             subject="recursiveRef",
+            case_description=(
+                "$recursiveRef with no $recursiveAnchor in "
+                "the initial target schema resource"
+            ),
+            description=(
+                "leaf node does not match: recursion uses the inner schema"
+            ),
         )(test)
         or skip(
-            message="These tests depends on dynamicRef working.",
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description="leaf node matches: recursion uses the inner schema",
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            case_description=(
+                "dynamic $recursiveRef destination (not predictable "
+                "at schema compile time)"
+            ),
+            description="integer node",
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            case_description=(
+                "multiple dynamic paths to the $recursiveRef keyword"
+            ),
+            description="recurse to integerNode - floats are not allowed",
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description="integer does not match as a property value",
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description=(
+                "leaf node does not match: "
+                "recursion only uses inner schema"
+            ),
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description=(
+                "leaf node matches: "
+                "recursion only uses inner schema"
+            ),
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description=(
+                "two levels, integer does not match as a property value"
+            ),
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description="recursive mismatch",
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
+            subject="recursiveRef",
+            description="two levels, no match",
+        )(test)
+        or skip(
+            message="recursiveRef support isn't working yet.",
             subject="id",
             case_description=(
                 "Invalid use of fragments in location-independent $id"
             ),
         )(test)
         or skip(
-            message="These tests depends on dynamicRef working.",
+            message="dynamicRef support isn't fully working yet.",
             subject="defs",
             description="invalid definition schema",
         )(test)
         or skip(
-            message="These tests depends on dynamicRef working.",
+            message="dynamicRef support isn't fully working yet.",
             subject="anchor",
             case_description="same $anchor with different base uri",
         )(test)
         or skip(
-            message="Vocabulary support is not yet present.",
+            message="Vocabulary support is still in-progress.",
             subject="vocabulary",
+            description=(
+                "no validation: invalid number, but it still validates"
+            ),
         )(test)
         or skip(
             message=bug(),
@@ -376,6 +396,11 @@ TestDraft201909 = DRAFT201909.to_unittest_testcase(
                 "not just immediate parent"
             ),
         )(test)
+        or skip(
+            message=bug(),
+            subject="refRemote",
+            case_description="remote HTTP ref with nested absolute ref",
+        )(test)
     ),
 )
 
@@ -383,12 +408,12 @@ TestDraft201909 = DRAFT201909.to_unittest_testcase(
 TestDraft201909Format = DRAFT201909.to_unittest_testcase(
     DRAFT201909.format_tests(),
     Validator=jsonschema.Draft201909Validator,
-    format_checker=jsonschema.draft201909_format_checker,
+    format_checker=jsonschema.Draft201909Validator.FORMAT_CHECKER,
     skip=lambda test: (
         complex_email_validation(test)
         or allowed_leading_zeros(test)
         or leap_second(test)
-        or missing_format(jsonschema.draft201909_format_checker)(test)
+        or missing_format(jsonschema.Draft201909Validator)(test)
         or complex_email_validation(test)
     ),
 )
@@ -397,6 +422,7 @@ TestDraft201909Format = DRAFT201909.to_unittest_testcase(
 TestDraft202012 = DRAFT202012.to_unittest_testcase(
     DRAFT202012.tests(),
     DRAFT202012.optional_tests_of(name="bignum"),
+    DRAFT202012.optional_tests_of(name="cross-draft"),
     DRAFT202012.optional_tests_of(name="float-overflow"),
     DRAFT202012.optional_tests_of(name="non-bmp-regex"),
     DRAFT202012.optional_tests_of(name="refOfUnknownKeyword"),
@@ -404,21 +430,88 @@ TestDraft202012 = DRAFT202012.to_unittest_testcase(
     skip=lambda test: (
         narrow_unicode_build(test)
         or skip(
-            message="dynamicRef support isn't working yet.",
+            message="dynamicRef support isn't fully working yet.",
             subject="dynamicRef",
+            description="The recursive part is not valid against the root",
         )(test)
         or skip(
-            message="These tests depends on dynamicRef working.",
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description="incorrect extended schema",
+            case_description=(
+                "$ref and $dynamicAnchor are independent of order - "
+                "$defs first"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description="correct extended schema",
+            case_description=(
+                "$ref and $dynamicAnchor are independent of order - "
+                "$defs first"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description="correct extended schema",
+            case_description=(
+                "$ref and $dynamicAnchor are independent of order - $ref first"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description="incorrect extended schema",
+            case_description=(
+                "$ref and $dynamicAnchor are independent of order - $ref first"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description=(
+                "/then/$defs/thingy is the final stop for the $dynamicRef"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description=(
+                "string matches /$defs/thingy, but the $dynamicRef "
+                "does not stop here"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description=(
+                "string matches /$defs/thingy, but the $dynamicRef "
+                "does not stop here"
+            ),
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
+            subject="dynamicRef",
+            description="recurse to integerNode - floats are not allowed",
+        )(test)
+        or skip(
+            message="dynamicRef support isn't fully working yet.",
             subject="defs",
+            description="invalid definition schema",
         )(test)
         or skip(
-            message="These tests depends on dynamicRef working.",
+            message="dynamicRef support isn't fully working yet.",
             subject="anchor",
             case_description="same $anchor with different base uri",
         )(test)
         or skip(
-            message="Vocabulary support is not yet present.",
+            message="Vocabulary support is still in-progress.",
             subject="vocabulary",
+            description=(
+                "no validation: invalid number, but it still validates"
+            ),
         )(test)
         or skip(
             message=bug(),
@@ -428,6 +521,11 @@ TestDraft202012 = DRAFT202012.to_unittest_testcase(
                 "not just immediate parent"
             ),
         )(test)
+        or skip(
+            message=bug(),
+            subject="refRemote",
+            case_description="remote HTTP ref with nested absolute ref",
+        )(test)
     ),
 )
 
@@ -435,12 +533,12 @@ TestDraft202012 = DRAFT202012.to_unittest_testcase(
 TestDraft202012Format = DRAFT202012.to_unittest_testcase(
     DRAFT202012.format_tests(),
     Validator=jsonschema.Draft202012Validator,
-    format_checker=jsonschema.draft202012_format_checker,
+    format_checker=jsonschema.Draft202012Validator.FORMAT_CHECKER,
     skip=lambda test: (
         complex_email_validation(test)
         or allowed_leading_zeros(test)
         or leap_second(test)
-        or missing_format(jsonschema.draft202012_format_checker)(test)
+        or missing_format(jsonschema.Draft202012Validator)(test)
         or complex_email_validation(test)
     ),
 )
